@@ -1,9 +1,6 @@
 #![feature(repr_simd)]
-#![feature(slice_sort_by_cached_key)]
 #![feature(test)]
 
-extern crate rand;
-extern crate rand_xorshift;
 extern crate test;
 
 mod btree;
