@@ -155,6 +155,11 @@ fn verify_all_signatures() {
             // https://github.com/rust-lang/rfcs/issues/2512
             "ud2"
                 => continue,
+            // Intel requires the mask argument for _mm_shuffle_ps to be an
+            // unsigned integer, but all other _mm_shuffle_.. intrinsics
+            // take a signed-integer. This breaks `_MM_SHUFFLE` for
+            // `_mm_shuffle_ps`:
+            "_mm_shuffle_ps" => continue,
             _ => {}
         }
 
